@@ -233,7 +233,8 @@ ORBit_marshal_value (GIOPSendBuffer *buf,
 		case CORBA_tk_boolean:
 		case CORBA_tk_char:
 		case CORBA_tk_octet:
-			giop_send_buffer_append (buf, val, tc->length);
+			giop_send_buffer_append (buf, *val, tc->length);
+			*val = ((guchar *)*val) + tc->length;
 			break;
 		default: {
 			int align = tc->subtypes[0]->c_align;
@@ -292,8 +293,8 @@ ORBit_get_union_switch (CORBA_TypeCode  tc,
 
 		if (update)
 			*val = ((guchar *)*val) + sizeof (CORBA_long);
-		}
 		break;
+	}
 	case CORBA_tk_ushort:
 	case CORBA_tk_short: {
 		CORBA_short tmp;
@@ -303,16 +304,18 @@ ORBit_get_union_switch (CORBA_TypeCode  tc,
 
 		if (update)
 			*val = ((guchar *)*val) + sizeof (CORBA_short);
-		}
 		break;
+	}
 	case CORBA_tk_char:
 	case CORBA_tk_boolean:
 	case CORBA_tk_octet:
 		retval = *(CORBA_octet *)*val;
-		if (update) *val = ((guchar *)*val) + sizeof (CORBA_char);
+		if (update)
+			*val = ((guchar *)*val) + sizeof (CORBA_char);
 		break;
 	default:
-		g_error ("Wow, some nut has passed us a weird type[%d] as a union discriminator!", tc->kind);
+		g_error ("Wow, some nut has passed us a weird "
+			 "type[%d] as a union discriminator!", tc->kind);
 	}
 
 	return retval;
@@ -380,10 +383,10 @@ ORBit_marshal_any (GIOPSendBuffer *buf, const CORBA_any *val)
 /* FIXME: we need two versions of this - one for swap
  * endianness, and 1 for not */
 gboolean
-ORBit_demarshal_value (CORBA_TypeCode tc,
-		      gpointer *val,
-		      GIOPRecvBuffer *buf,
-		      CORBA_ORB orb)
+ORBit_demarshal_value (CORBA_TypeCode  tc,
+		       gpointer       *val,
+		       GIOPRecvBuffer *buf,
+		       CORBA_ORB       orb)
 {
 	CORBA_long i;
 
@@ -409,8 +412,7 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 	}
 	case CORBA_tk_long:
 	case CORBA_tk_ulong:
-	case CORBA_tk_enum:
-	{
+	case CORBA_tk_enum: {
 		CORBA_unsigned_long *ptr;
 		*val = ALIGN_ADDRESS (*val, ORBIT_ALIGNOF_CORBA_LONG);
 		buf->cur = ALIGN_ADDRESS (buf->cur, sizeof (CORBA_long));
@@ -422,11 +424,10 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 			*ptr = GUINT32_SWAP_LE_BE (*ptr);
 		buf->cur += sizeof (CORBA_long);
 		*val = ((guchar *)*val) + sizeof (CORBA_long);
+		break;
 	}
-	break;
 	case CORBA_tk_longlong:
-	case CORBA_tk_ulonglong:
-	{
+	case CORBA_tk_ulonglong: {
 		CORBA_unsigned_long_long *ptr;
 		*val = ALIGN_ADDRESS (*val, ORBIT_ALIGNOF_CORBA_LONG_LONG);
 		buf->cur = ALIGN_ADDRESS (buf->cur, sizeof (CORBA_long_long));
@@ -438,10 +439,9 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 			*ptr = GUINT64_SWAP_LE_BE (*ptr);
 		buf->cur += sizeof (CORBA_long_long);
 		*val = ((guchar *)*val) + sizeof (CORBA_long_long);
+		break;
 	}
-	break;
-	case CORBA_tk_longdouble:
-	{
+	case CORBA_tk_longdouble: {
 		CORBA_long_double *ptr;
 		*val = ALIGN_ADDRESS (*val, ORBIT_ALIGNOF_CORBA_LONG_DOUBLE);
 		buf->cur = ALIGN_ADDRESS (buf->cur, sizeof (CORBA_long_double));
@@ -454,8 +454,8 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 			*ptr = *(CORBA_long_double *)buf->cur;
 		buf->cur += sizeof (CORBA_long_double);
 		*val = ((guchar *)*val) + sizeof (CORBA_long_double);
+		break;
 	}
-	break;
 	case CORBA_tk_float: {
 		CORBA_float *ptr;
 		*val = ALIGN_ADDRESS (*val, ORBIT_ALIGNOF_CORBA_FLOAT);
@@ -577,8 +577,8 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 
 		/* WATCHOUT: end subtc body may not be end of union */
 		*val = ((guchar *)*val) + sz;
+		break;
 	}
-	break;
 	case CORBA_tk_string:
 	case CORBA_tk_wstring:
 		*val = ALIGN_ADDRESS (*val, ORBIT_ALIGNOF_CORBA_POINTER);
@@ -611,26 +611,42 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 		else
 			p->_length = *(CORBA_unsigned_long *)buf->cur;
 		buf->cur += sizeof (CORBA_long);
-		if (tc->subtypes[0]->kind == CORBA_tk_octet
-		    || tc->subtypes[0]->kind == CORBA_tk_boolean
-		    || tc->subtypes[0]->kind == CORBA_tk_char) {
+		if (tc->subtypes[0]->kind == CORBA_tk_octet ||
+		    tc->subtypes[0]->kind == CORBA_tk_boolean ||
+		    tc->subtypes[0]->kind == CORBA_tk_char) {
 			/* This special-casing could be taken further to apply to
 			   all atoms... */
-			if ((buf->cur + p->_length) > buf->end
-			    || (buf->cur + p->_length) < buf->cur)
+			if ((buf->cur + p->_length) > buf->end ||
+			    (buf->cur + p->_length) < buf->cur)
 				return TRUE;
 			p->_buffer = ORBit_alloc_simple (p->_length);
 			memcpy (p->_buffer, buf->cur, p->_length);
 			buf->cur = ((guchar *)buf->cur) + p->_length;
 		} else {
+			CORBA_unsigned_long alloc = 4096;
+
 			p->_buffer = ORBit_alloc_tcval (tc->subtypes[0],
-							p->_length);
+							MIN (p->_length, alloc));
 			subval = p->_buffer;
 
-			for (i = 0; i < p->_length; i++)
+			for (i = 0; i < p->_length; i++) {
+				if (i == alloc) {
+					size_t delta = (guchar *)subval - (guchar *)p->_buffer;
+
+					p->_buffer = ORBit_realloc_tcval (
+						p->_buffer, tc->subtypes [0],
+						alloc, MIN (alloc * 2, p->_length));
+					alloc = alloc * 2; /* exponential */
+					subval = p->_buffer + delta;
+				}
+
 				if (ORBit_demarshal_value (tc->subtypes[0], &subval,
-							   buf, orb))
+							   buf, orb)) {
+					CORBA_free (p->_buffer);
+					p->_buffer = NULL;
 					return TRUE;
+				}
+			}
 		}
 
 		*val = ((guchar *)*val) + sizeof (CORBA_sequence_CORBA_octet);
@@ -650,7 +666,6 @@ ORBit_demarshal_value (CORBA_TypeCode tc,
 	return FALSE;
 }
 
-
 gpointer
 ORBit_demarshal_arg (GIOPRecvBuffer *buf,
 		     CORBA_TypeCode tc,
@@ -700,7 +715,6 @@ CORBA_any__freekids (gpointer mem, gpointer dat)
 	return t + 1;
 }
 
-
 CORBA_any *
 CORBA_any__alloc (void)
 {
@@ -774,8 +788,7 @@ ORBit_copy_value_core (gconstpointer *val,
 		*val = ((guchar *)*val) + sizeof (CORBA_octet);
 		*newval = ((guchar *)*newval) + sizeof (CORBA_octet);
 		break;
-	case CORBA_tk_any:
-	{
+	case CORBA_tk_any: {
 		const CORBA_any *oldany;
 		CORBA_any *newany;
 		*val = ALIGN_ADDRESS (*val, ORBIT_ALIGNOF_CORBA_ANY);
@@ -783,14 +796,12 @@ ORBit_copy_value_core (gconstpointer *val,
 		oldany = *val;
 		newany = *newval;
 		newany->_type = ORBit_RootObject_duplicate (oldany->_type);
-		/* XXX are we supposed to do above even if oldany->_release
-		   == FALSE? */
 		newany->_value = ORBit_copy_value (oldany->_value, oldany->_type);
 		newany->_release = CORBA_TRUE;
 		*val = ((guchar *)*val) + sizeof (CORBA_any);
 		*newval = ((guchar *)*newval) + sizeof (CORBA_any);
+		break;
 	}
-	break;
 	case CORBA_tk_Principal:
 		*val = ALIGN_ADDRESS (*val,
 				     MAX (MAX (ORBIT_ALIGNOF_CORBA_LONG,
@@ -825,8 +836,7 @@ ORBit_copy_value_core (gconstpointer *val,
 		for (i = 0; i < tc->sub_parts; i++)
 			ORBit_copy_value_core (val, newval, tc->subtypes[i]);
 		break;
-	case CORBA_tk_union:
-	{
+	case CORBA_tk_union: {
 		CORBA_TypeCode utc = ORBit_get_union_tag (tc, (gconstpointer *)val, FALSE);
 		gint	       union_align = tc->c_align;
 		gint	       discrim_align = MAX (tc->discriminator->c_align, tc->c_align);
