@@ -8,41 +8,42 @@
 #include "orbit-poa.h"
 #include "../orb-core/orbit-debug.h"
 
-int
+CORBA_long
 ORBit_adaptor_setup (ORBit_ObjectAdaptor adaptor,
 		     CORBA_ORB           orb)
 {
+	int adaptor_id;
 	CORBA_long *tptr;
-	int         adaptor_id;
 
 	adaptor_id = orb->adaptors->len;
 
 	g_ptr_array_set_size (orb->adaptors, adaptor_id + 1);
 	g_ptr_array_index    (orb->adaptors, adaptor_id) = adaptor;
 
-	adaptor->adaptor_key._length  = ORBIT_ADAPTOR_KEY_LEN;
+	adaptor->adaptor_key._length  = ORBIT_ADAPTOR_PREFIX_LEN;
 	adaptor->adaptor_key._buffer  =
-		CORBA_sequence_CORBA_octet_allocbuf (ORBIT_ADAPTOR_KEY_LEN);
+		CORBA_sequence_CORBA_octet_allocbuf (adaptor->adaptor_key._length);
 	adaptor->adaptor_key._release = CORBA_TRUE;
 
+	ORBit_genuid_buffer (adaptor->adaptor_key._buffer + sizeof (CORBA_long),
+			     ORBIT_ADAPTOR_KEY_LEN, ORBIT_GENUID_COOKIE);
+
 	tptr = (CORBA_long *) adaptor->adaptor_key._buffer;
 	*tptr = adaptor_id;
-	ORBit_genuid_buffer (adaptor->adaptor_key._buffer + sizeof (CORBA_long),
-			     ORBIT_RAND_DATA_LEN);
 
-	return adaptor_id;
+	return *tptr;
 }
 
 static ORBit_ObjectAdaptor
 ORBit_adaptor_find (CORBA_ORB orb, ORBit_ObjectKey *objkey)
 {
-	ORBit_ObjectAdaptor   adaptor;
-	gint32                adaptorId;
+	gint32 adaptorId;
+	ORBit_ObjectAdaptor adaptor;
 
 	if (!objkey)
 		return NULL;
 
-	if (objkey->_length < ORBIT_ADAPTOR_KEY_LEN)
+	if (objkey->_length < ORBIT_ADAPTOR_PREFIX_LEN)
 		return NULL;
 
 	memcpy (&adaptorId, objkey->_buffer, sizeof (gint32));
@@ -52,8 +53,9 @@ ORBit_adaptor_find (CORBA_ORB orb, ORBit_ObjectKey *objkey)
 
 	adaptor = g_ptr_array_index (orb->adaptors, adaptorId);
 
-	if (memcmp (objkey->_buffer, adaptor->adaptor_key._buffer,
-	            ORBIT_ADAPTOR_KEY_LEN))
+	if (memcmp (objkey->_buffer,
+		    adaptor->adaptor_key._buffer,
+	            ORBIT_ADAPTOR_PREFIX_LEN))
 		return NULL;
 
 	return adaptor;
@@ -62,21 +64,35 @@ ORBit_adaptor_find (CORBA_ORB orb, ORBit_ObjectKey *objkey)
 void
 ORBit_handle_request (CORBA_ORB orb, GIOPRecvBuffer *recv_buffer)
 {
-	ORBit_ObjectAdaptor  adaptor;
-	ORBit_ObjectKey     *objkey;
+	ORBit_ObjectKey *objkey;
+	ORBit_ObjectAdaptor adaptor;
 
 	objkey = giop_recv_buffer_get_objkey (recv_buffer);
 	adaptor = ORBit_adaptor_find (orb, objkey);
 
 	if (!adaptor || !objkey) {
+		CORBA_Environment env;
+
+		CORBA_exception_init (&env);
+
 		tprintf ("Error: failed to find adaptor or objkey for "
 			 "object while invoking method '%s'",
 			 giop_recv_buffer_get_opname (recv_buffer));
-		return;
+		
+		CORBA_exception_set_system (
+			&env, ex_CORBA_OBJECT_NOT_EXIST, 
+			CORBA_COMPLETED_NO);
+		ORBit_recv_buffer_return_sys_exception (recv_buffer, &env);
+
+		CORBA_exception_free (&env);
+
+	} else {
+		dprintf (MESSAGES, "p %d: handle request '%s'\n",
+			 getpid (),
+			 giop_recv_buffer_get_opname (recv_buffer));
+
+		adaptor->handle_request (adaptor, recv_buffer, objkey);
 	}
-	dprintf (MESSAGES, "p %d: handle request '%s'\n", getpid (),
-		 giop_recv_buffer_get_opname (recv_buffer));
-	adaptor->handle_request (adaptor, recv_buffer, objkey);
 }
 
 void
